/* BdfServer_Html - Copyright (c) 2018-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.html;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.storage.JsAnalyser;
import fr.exemole.bdfserver.api.storage.ResourceStorages;
import fr.exemole.bdfserver.html.jslib.JsLib;
import fr.exemole.bdfserver.html.jslib.JsrenderTemplateFamily;
import fr.exemole.bdfserver.html.jslib.ThirdLib;
import fr.exemole.bdfserver.tools.BdfServerUtils;
import fr.exemole.bdfserver.tools.docstream.DocStreamFactory;
import fr.exemole.bdfserver.tools.storage.StorageUtils;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;
import net.fichotheque.Fichotheque;
import net.fichotheque.addenda.Addenda;
import net.fichotheque.album.Album;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.metadata.FichothequeMetadata;
import net.fichotheque.sphere.Sphere;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.utils.FichothequeUtils;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.html.HtmlPrinter;
import net.mapeadores.util.html.PageHtmlPrinter;
import net.mapeadores.util.html.TrustedHtmlFactory;
import net.mapeadores.util.io.DocStream;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LocalisationUtils;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelUtils;
import net.mapeadores.util.text.Labels;
import net.mapeadores.util.text.Phrase;
import net.mapeadores.util.text.RelativePath;


/**
 *
 * @author Vincent Calame
 */
public final class JsLibManager {

    private final static RelativePath BDFCORE = RelativePath.build("js/bdf/BDF.js");
    private final JsAnalyser jsAnalyser;
    private final Set<RelativePath> jsScriptSet = new LinkedHashSet();
    private final Map<String, ThirdLibBuffer> thirdLibMap = new HashMap<String, ThirdLibBuffer>();
    private final SortedMap<String, JsrenderTemplateFamily> templateMap = new TreeMap<String, JsrenderTemplateFamily>();
    private final Set<String> messageKeySet = new LinkedHashSet();
    private final Set<JsLib> jsLibSet = new HashSet<JsLib>();
    private String mainStorageKey = null;

    public JsLibManager(JsAnalyser jsAnalyser) {
        this.jsAnalyser = jsAnalyser;
        messageKeySet.addAll(jsAnalyser.getJsMessageKeySet(BDFCORE));
    }

    public void setMainStorageKey(String mainStorageKey) {
        this.mainStorageKey = mainStorageKey;
    }

    public void addJsLib(JsLib jsLib) {
        if (jsLibSet.contains(jsLib)) {
            return;
        } else {
            jsLibSet.add(jsLib);
        }
        for (JsLib dependency : jsLib.getDependencyList()) {
            addJsLib(dependency);
        }
        for (RelativePath relativePath : jsLib.getJsScriptList()) {
            if (!jsScriptSet.contains(relativePath)) {
                jsScriptSet.add(relativePath);
                messageKeySet.addAll(jsAnalyser.getJsMessageKeySet(relativePath));
            }
        }
        for (ThirdLib thirdLib : jsLib.getThirdLibList()) {
            String name = thirdLib.getName();
            ThirdLibBuffer buffer = thirdLibMap.get(name);
            if (buffer == null) {
                buffer = new ThirdLibBuffer(name);
                thirdLibMap.put(name, buffer);
            }
            buffer.addAll(thirdLib.getExtensionNameList());
        }
        for (JsrenderTemplateFamily templateFamily : jsLib.getJsrenderTemplateFamilyList()) {
            templateMap.put(templateFamily.getName(), templateFamily);
        }
    }

    public void resolve(PageHtmlPrinter htmlPrinter, String resourcePath, String suffix) {
        htmlPrinter
                .addJsUrl(resourcePath + "third-lib/jquery/jquery.js" + suffix);
        ThirdLibBuffer jqueryThirdLib = thirdLibMap.get(JsLib.JQUERY);
        if (jqueryThirdLib != null) {
            for (String jqueryExtensionName : jqueryThirdLib.extensionNameSet) {
                htmlPrinter
                        .addJsUrl(resourcePath + "third-lib/jquery/jquery." + jqueryExtensionName + ".js" + suffix);
                if (jqueryExtensionName.equals("ui")) {
                    htmlPrinter
                            .addCssUrl(resourcePath + "third-lib/jquery/ui/jquery-ui.css" + suffix);
                }
            }
        }
        if (templateMap.size() > 0) {
            htmlPrinter
                    .addJsUrl(resourcePath + "third-lib/jsrender/jsrender.min.js" + suffix);
        }
        ThirdLibBuffer codemirrorThirdLib = thirdLibMap.get(JsLib.CODEMIRROR);
        if (codemirrorThirdLib != null) {
            htmlPrinter
                    .addJsUrl(resourcePath + "third-lib/codemirror/lib/codemirror.js" + suffix)
                    .addCssUrl(resourcePath + "third-lib/codemirror/lib/codemirror.css" + suffix);
            for (String codemirrorMode : codemirrorThirdLib.extensionNameSet) {
                if (isBdfMode(codemirrorMode)) {
                    htmlPrinter
                            .addJsUrl(resourcePath + "js/codemirror/" + codemirrorMode + ".js" + suffix);
                } else {
                    htmlPrinter
                            .addJsUrl(resourcePath + "third-lib/codemirror/mode/" + codemirrorMode + "/" + codemirrorMode + ".js" + suffix);
                }
            }
        }
        ThirdLibBuffer leafletThirdLib = thirdLibMap.get(JsLib.LEAFLET);
        if (leafletThirdLib != null) {
            htmlPrinter
                    .addJsUrl(resourcePath + "third-lib/leaflet/leaflet.js" + suffix)
                    .addCssUrl(resourcePath + "third-lib/leaflet/leaflet.css" + suffix);
            for (String leafletExtension : leafletThirdLib.extensionNameSet) {
                htmlPrinter
                        .addJsUrl(resourcePath + "third-lib/leaflet/leaflet." + leafletExtension + ".js" + suffix)
                        .addCssUrl(resourcePath + "third-lib/leaflet/leaflet." + leafletExtension + ".css" + suffix);
            }

        }
        ThirdLibBuffer mousetrapThirdLib = thirdLibMap.get(JsLib.MOUSETRAP);
        if (mousetrapThirdLib != null) {
            htmlPrinter
                    .addJsUrl(resourcePath + "third-lib/mousetrap/mousetrap.js" + suffix);
        }
        ThirdLibBuffer mergelyThirdLib = thirdLibMap.get(JsLib.MERGELY);
        if (mergelyThirdLib != null) {
            htmlPrinter
                    .addJsUrl(resourcePath + "third-lib/mergely/mergely.js" + suffix)
                    .addCssUrl(resourcePath + "third-lib/mergely/mergely.css" + suffix);
        }
        ThirdLibBuffer jsrenderThirdLib = thirdLibMap.get(JsLib.JSRENDER);
        if (jsrenderThirdLib != null) {
            htmlPrinter
                    .addJsUrl(resourcePath + "third-lib/jsrender/jsrender.min.js" + suffix);
        }
        htmlPrinter
                .addJsUrl(resourcePath + "js/overlay/Overlay.js" + suffix)
                .addJsUrl(resourcePath + BDFCORE.toString() + suffix);
        for (RelativePath relativePath : jsScriptSet) {
            htmlPrinter
                    .addJsUrl(resourcePath + relativePath.toString() + suffix);
        }
    }

    public void end(HtmlPrinter hp, BdfServer bdfServer, Lang workingLang) {
        Fichotheque fichotheque = bdfServer.getFichotheque();
        FichothequeMetadata fichothequeMetadata = fichotheque.getFichothequeMetadata();
        hp
                .__breakLine()
                .SCRIPT()
                .__newLine()
                .__escape("BDF.BASE_URI=")
                .__scriptLiteral(fichothequeMetadata.getAuthority() + "/" + fichothequeMetadata.getBaseName())
                .__scriptLineEnd();
        if (mainStorageKey != null) {
            hp
                    .__escape("BDF.MAINSTORAGE_KEY=")
                    .__scriptLiteral(mainStorageKey)
                    .__scriptLineEnd();
        }
        hp
                .__escape("BDF.WORKING_LANG=")
                .__scriptLiteral(workingLang.toString())
                .__scriptLineEnd()
                ._SCRIPT();
        insertTemplates(hp, bdfServer.getResourceStorages());
        hp
                .SCRIPT()
                .__escape("BDF.Loc.add({")
                .__newLine();
        boolean next = false;
        for (String messageKey : messageKeySet) {
            switch (messageKey) {
                case "_ @title_corpus@": {
                    for (Corpus corpus : fichotheque.getCorpusList()) {
                        insertLoc(hp, "subset:" + corpus.getSubsetKey().toString(), FichothequeUtils.getTitle(corpus, workingLang), next);
                        next = true;
                    }
                    break;
                }
                case "_ @title_thesaurus@": {
                    for (Thesaurus thesaurus : fichotheque.getThesaurusList()) {
                        insertLoc(hp, "subset:" + thesaurus.getSubsetKey().toString(), FichothequeUtils.getTitle(thesaurus, workingLang), next);
                        next = true;
                    }
                    break;
                }
                case "_ @title_addenda@": {
                    for (Addenda addenda : fichotheque.getAddendaList()) {
                        insertLoc(hp, "subset:" + addenda.getSubsetKey().toString(), FichothequeUtils.getTitle(addenda, workingLang), next);
                        next = true;
                    }
                    break;
                }
                case "_ @title_album@": {
                    for (Album album : fichotheque.getAlbumList()) {
                        insertLoc(hp, "subset:" + album.getSubsetKey().toString(), FichothequeUtils.getTitle(album, workingLang), next);
                        next = true;
                    }
                    break;
                }
                case "_ @title_sphere@": {
                    for (Sphere sphere : fichotheque.getSphereList()) {
                        insertLoc(hp, "subset:" + sphere.getSubsetKey().toString(), FichothequeUtils.getTitle(sphere, workingLang), next);
                        next = true;
                    }
                    break;
                }
                case "_ @phrases@": {
                    Labels titleLabels = fichotheque.getFichothequeMetadata().getTitleLabels();
                    Label titleLabel = LabelUtils.getLangPartCheckedLabel(titleLabels, workingLang);
                    if (titleLabel == null) {
                        titleLabel = LabelUtils.getFirstLabel(titleLabels);
                    }
                    if (titleLabel != null) {
                        insertLoc(hp, "fichotheque", titleLabel.getLabelString(), next);
                        next = true;
                    }
                    for (Phrase phrase : fichotheque.getFichothequeMetadata().getPhrases()) {
                        String key = "phrase:" + phrase.getName();
                        Label label = LabelUtils.getLangPartCheckedLabel(phrase, workingLang);
                        if (label == null) {
                            label = LabelUtils.getFirstLabel(phrase);
                        }
                        if (label != null) {
                            insertLoc(hp, key, label.getLabelString(), next);
                            next = true;
                        }
                    }
                    break;
                }
                default: {
                    String loc = hp.getMessageLocalisation().toString(messageKey);
                    if (loc != null) {
                        insertLoc(hp, messageKey, loc, next);
                        next = true;
                    }
                }
            }
        }
        hp
                .__newLine()
                .__escape("});")
                ._SCRIPT();
    }

    public void end(HtmlPrinter hp, ResourceStorages resourceStorages, Lang workingLang) {
        hp
                .__breakLine()
                .SCRIPT()
                .__newLine();
        if (mainStorageKey != null) {
            hp
                    .__escape("BDF.MAINSTORAGE_KEY=")
                    .__scriptLiteral(mainStorageKey)
                    .__scriptLineEnd();
        }
        hp
                .__escape("BDF.WORKING_LANG=")
                .__scriptLiteral(workingLang.toString())
                .__scriptLineEnd()
                ._SCRIPT();
        insertTemplates(hp, resourceStorages);
        hp
                .SCRIPT()
                .__escape("BDF.Loc.add({")
                .__newLine();
        boolean next = false;
        for (String messageKey : messageKeySet) {
            String loc = hp.getMessageLocalisation().toString(messageKey);
            if (loc != null) {
                insertLoc(hp, messageKey, loc, next);
                next = true;
            }
        }
        hp
                .__newLine()
                .__escape("});")
                ._SCRIPT();
    }

    private void insertTemplates(HtmlPrinter hp, ResourceStorages resourceStorages) {
        for (JsrenderTemplateFamily templateFamily : templateMap.values()) {
            String familyName = templateFamily.getName();
            Set<String> existingSet = new HashSet<String>();
            RelativePath familyPath = getRelativePath(templateFamily);
            RelativePath customFamilyPath = RelativePath.build("custom/" + familyPath.getPath());
            SortedMap<String, RelativePath> customMap = StorageUtils.listResources(resourceStorages, customFamilyPath, true);
            if (!customMap.isEmpty()) {
                insertTemplates(hp, resourceStorages, familyName, customMap, existingSet);
            }
            SortedMap<String, RelativePath> map = StorageUtils.listResources(resourceStorages, familyPath, true);
            insertTemplates(hp, resourceStorages, familyName, map, existingSet);
        }
    }

    private void insertTemplates(HtmlPrinter hp, ResourceStorages resourceStorages, String familyName, SortedMap<String, RelativePath> map, Set<String> existingSet) {
        for (Map.Entry<String, RelativePath> entry : map.entrySet()) {
            String name = entry.getKey();
            if ((name.endsWith(".html")) && (!existingSet.contains(name))) {
                existingSet.add(name);
                name = name.substring(0, name.length() - 5);
                String content = "";
                DocStream docStream = DocStreamFactory.getResourceDocStream(resourceStorages, entry.getValue());
                if (docStream != null) {
                    content = BdfServerUtils.toString(docStream);
                    messageKeySet.addAll(LocalisationUtils.scanMessageKeys(content));
                }
                String templateName;
                if (familyName.isEmpty()) {
                    templateName = name;
                } else {
                    templateName = familyName + ":" + name;
                }
                hp
                        .SCRIPT(HA.attr("data-name", templateName).type("text/x-jsrender"))
                        .__append(TrustedHtmlFactory.UNCHECK.build(content))
                        ._SCRIPT();
            }
        }
    }

    private static RelativePath getRelativePath(JsrenderTemplateFamily templateFamily) {
        String familyName = templateFamily.getName();
        switch (templateFamily.getOriginType()) {
            case JsrenderTemplateFamily.CORE_TYPE:
                return RelativePath.build("templates/" + familyName);
            case JsrenderTemplateFamily.EXTENSION_TYPE:
                return StorageUtils.buildExtensionResourcePath(templateFamily.getOriginName(), "templates/" + familyName);
            case JsrenderTemplateFamily.APP_TYPE:
                String subpath;
                if (familyName.isEmpty()) {
                    subpath = "templates";
                } else {
                    subpath = "lib/templates/" + familyName;
                }
                return StorageUtils.buildAppResourcePath(templateFamily.getOriginName(), subpath);
            default:
                throw new SwitchException("Unkown type: " + templateFamily.getOriginType());
        }
    }

    private void insertLoc(HtmlPrinter hp, String name, String value, boolean next) {
        if (next) {
            hp
                    .__escape(',')
                    .__newLine();
        }
        hp
                .__scriptLiteral(name)
                .__escape(':')
                .__scriptLiteral(value);
    }

    private static boolean isBdfMode(String mode) {
        switch (mode) {
            case "attributes":
            case "ficheblock":
            case "subsettree":
            case "tableexport":
            case "uicomponents":
                return true;
            default:
                return false;
        }
    }


    private static class ThirdLibBuffer {

        private final String name;
        private final Set<String> extensionNameSet = new LinkedHashSet<String>();

        private ThirdLibBuffer(String name) {
            this.name = name;
        }

        private void addAll(Collection<String> extensionNames) {
            extensionNameSet.addAll(extensionNames);
        }

    }

}
