/* BdfServer_HtmlProducers - Copyright (c) 2010-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.configuration;

import fr.exemole.bdfserver.api.instruction.BdfParameters;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.api.interaction.InteractionConstants;
import fr.exemole.bdfserver.api.interaction.domains.ConfigurationDomain;
import fr.exemole.bdfserver.commands.administration.ResourceUploadCommand;
import fr.exemole.bdfserver.html.BdfServerHtmlProducer;
import fr.exemole.bdfserver.html.consumers.Grid;
import fr.exemole.bdfserver.html.consumers.PageUnit;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import fr.exemole.bdfserver.tools.storage.IconScanEngine;
import fr.exemole.bdfserver.tools.storage.StorageUtils;
import java.util.List;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.html.CommandBox;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.html.HtmlAttributes;
import net.mapeadores.util.text.RelativePath;


/**
 *
 * @author Vincent Calame
 */
public class LogosFormHtmlProducer extends BdfServerHtmlProducer {

    public LogosFormHtmlProducer(BdfParameters bdfParameters) {
        super(bdfParameters);
        addThemeCss("configuration.css");
    }

    @Override
    public void printHtml() {
        start();
        printCommandMessageUnit();
        CommandBox commandBox = CommandBox.init()
                .action(Domains.CONFIGURATION)
                .family("CNF")
                .page(ConfigurationDomain.LOGOSFORM_PAGE)
                .name(BdfInstructionUtils.getAbsoluteCommandName(Domains.ADMINISTRATION, ResourceUploadCommand.COMMANDNAME))
                .multipart(true)
                .actionCssClass("action-Save")
                .submitLocKey("_ submit.administration.resourceupload");
        printLogoBox(commandBox);
        printIconBox(commandBox, "16");
        printIconBox(commandBox, "32");
        printExistingCustomIconUnit();
        ConfigurationCommandBoxUtils.printCustomIconUploadBox(this, commandBox);
        end();
    }

    private void printIconBox(CommandBox commandBox, String size) {
        RelativePath relativePath = getRelativePath(size);
        boolean containsIcon = StorageUtils.containsVarResource(bdfServer, relativePath);
        commandBox
                .lockey(getCommandLockKey(size))
                .hidden(InteractionConstants.PATH_PARAMNAME, relativePath.toString());
        this
                .__start(commandBox)
                .__(Grid.START)
                .__(printImageRow(containsIcon, HA.src(relativePath.toString()).alt("icon " + size + "x" + size).classes("configuration-Logo").height(size).width(size)))
                .__(printFileInput("_ label.configuration.file_icon"))
                .__(Grid.END)
                .__end(commandBox);
    }

    private boolean printLogoBox(CommandBox commandBox) {
        boolean containsLogo = bdfServer.getResourceStorages().containsResource(StorageUtils.LOGO);
        commandBox
                .lockey(ResourceUploadCommand.LOGO_COMMANDKEY)
                .hidden(InteractionConstants.PATH_PARAMNAME, StorageUtils.LOGO.toString());
        this
                .__start(commandBox)
                .__(Grid.START)
                .__(printLogoRow(containsLogo, HA.src("images/logo.png").alt("logo").classes("configuration-Logo")))
                .__(printFileInput("_ label.configuration.file_logo"))
                .__(Grid.END)
                .__end(commandBox);
        return true;
    }

    private boolean printImageRow(boolean contains, HtmlAttributes imgAttributes) {
        String locKey = (contains) ? "_ label.configuration.icon_current" : "_ label.configuration.icon_default";
        this
                .P("grid-Row")
                .__(Grid.printLabelCells(this, locKey))
                .SPAN("grid-InputCell")
                .IMG(imgAttributes)
                ._SPAN()
                ._P();
        return true;
    }

    private boolean printLogoRow(boolean contains, HtmlAttributes imgAttributes) {
        if (contains) {
            this
                    .P("grid-Row")
                    .__(Grid.printLabelCells(this, "_ label.configuration.logo_current"))
                    .SPAN("grid-InputCell")
                    .IMG(imgAttributes)
                    ._SPAN()
                    ._P();
        } else {
            this
                    .P("grid-Row")
                    .SPAN("grid-UniqueCell")
                    .__localize("_ info.configuration.nologo")
                    ._SPAN()
                    ._P();
        }
        return true;
    }

    private boolean printFileInput(String messageKey) {
        this
                .__(Grid.fileInputRow(messageKey, name(ResourceUploadCommand.FILE_PARAMNAME).size("75").classes("global-FileInput")));
        return true;
    }

    private boolean printExistingCustomIconUnit() {
        List<HtmlAttributes> existingList = IconScanEngine.run(bdfServer.getResourceStorages(), bdfServer.getMimeTypeResolver());
        this
                .__(PageUnit.start("action-Logos", "_ title.configuration.customincons"));
        if (existingList.isEmpty()) {
            this
                    .P()
                    .__localize("_ info.configuration.empty_customincons")
                    .P();
        } else {
            this
                    .__(Grid.START);
            for (HtmlAttributes htmlAttributes : existingList) {
                HtmlAttributes imgAttributes = HA.src(htmlAttributes.href()).classes("configuration-Logo");
                String labelText;
                String sizes = htmlAttributes.attr("sizes");
                if (sizes != null) {
                    labelText = sizes + " (" + htmlAttributes.rel() + ")";
                } else {
                    labelText = htmlAttributes.rel();
                }
                this
                        .P("grid-Row")
                        .SPAN("grid-LabelCell")
                        .__escape(labelText)
                        ._SPAN()
                        .SPAN("grid-ColonCell")
                        .__colon()
                        ._SPAN()
                        .SPAN("grid-InputCell")
                        .IMG(imgAttributes)
                        ._SPAN()
                        ._P();
            }
            this
                    .__(Grid.END);

        }
        this
                .__(PageUnit.END);
        return true;
    }


    private static String getCommandLockKey(String size) {
        switch (size) {
            case "16":
                return ResourceUploadCommand.ICON16_COMMANDKEY;
            case "32":
                return ResourceUploadCommand.ICON32_COMMANDKEY;
            default:
                throw new SwitchException("Unknown size: " + size);
        }
    }

    private static RelativePath getRelativePath(String size) {
        switch (size) {
            case "16":
                return StorageUtils.ICON;
            case "32":
                return StorageUtils.ICON32;
            default:
                throw new SwitchException("Unknown size: " + size);
        }
    }

}
