/* BdfServer_HtmlProducers - Copyright (c) 2012-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.corpus;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.ficheform.FormElementProvider;
import fr.exemole.bdfserver.api.instruction.BdfCommandResult;
import fr.exemole.bdfserver.api.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.instruction.BdfParameters;
import fr.exemole.bdfserver.api.instruction.OutputParameters;
import fr.exemole.bdfserver.api.instruction.PermissionBdfInstructionException;
import fr.exemole.bdfserver.api.interaction.domains.CorpusDomain;
import fr.exemole.bdfserver.api.managers.UiManager;
import fr.exemole.bdfserver.api.ui.UiComponents;
import fr.exemole.bdfserver.html.BdfServerHtmlProducer;
import fr.exemole.bdfserver.htmlproducers.edition.EditionHtmlProducerFactory;
import fr.exemole.bdfserver.tools.ficheform.FicheFormParametersBuilder;
import fr.exemole.bdfserver.tools.ficheform.FormElementProviderFactory;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import fr.exemole.bdfserver.tools.roles.PermissionUtils;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.pointeurs.FichePointeur;
import net.fichotheque.tools.duplication.SubsetMatch;
import net.fichotheque.tools.reponderation.ReponderationLog;
import net.fichotheque.utils.pointeurs.PointeurFactory;
import net.mapeadores.util.html.HtmlProducer;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public final class CorpusHtmlProducerFactory {

    private final static int CORPUS_ADMIN = 1;
    private final static int FICHE = 3;

    private CorpusHtmlProducerFactory() {
    }

    public static HtmlProducer getHtmlProducer(OutputParameters parameters) throws BdfInstructionException {
        BdfServerHtmlProducer htmlProducer = getBdfHtmlProducer(parameters);
        if (htmlProducer != null) {
            htmlProducer.setBdfCommandResult(parameters.getBdfCommandResult());
        }
        return htmlProducer;
    }

    private static BdfServerHtmlProducer getBdfHtmlProducer(OutputParameters parameters) throws BdfInstructionException {
        String page = parameters.getOutput();
        BdfServer bdfServer = parameters.getBdfServer();
        RequestMap requestMap = parameters.getRequestMap();
        BdfCommandResult bdfCommandResult = parameters.getBdfCommandResult();
        switch (getPageType(page)) {
            case CORPUS_ADMIN: {
                Corpus corpus = getCorpus(bdfServer, requestMap, bdfCommandResult);
                parameters.checkSubsetAdmin(corpus);
                switch (page) {
                    case CorpusDomain.CORPUS_PAGE: {
                        return new CorpusHtmlProducer(parameters, corpus);
                    }
                    case CorpusDomain.CORPUS_ADVANCEDCOMMANDS_PAGE: {
                        return new CorpusAdvancedCommandsHtmlProducer(parameters, corpus);
                    }
                    case CorpusDomain.CORPUS_DIAGRAM_PAGE: {
                        return new CorpusDiagramHtmlProducer(parameters, corpus);
                    }
                    case CorpusDomain.CORPUS_PHRASESFORM_PAGE: {
                        return new CorpusPhrasesFormHtmlProducer(parameters, corpus);
                    }
                    case CorpusDomain.FIELD_CREATIONFORM_PAGE: {
                        return new FieldCreationFormHtmlProducer(parameters, corpus);
                    }
                    case CorpusDomain.FIELD_OPTIONSFORM_PAGE: {
                        return new FieldOptionsFormHtmlProducer(parameters, corpus);
                    }
                    case CorpusDomain.FIELD_REMOVEFORM_PAGE: {
                        return new FieldRemoveFormHtmlProducer(parameters, corpus);
                    }
                    case CorpusDomain.INCLUDE_ADDENDAFORM_PAGE: {
                        return new IncludeAddendaFormHtmlProducer(parameters, corpus);
                    }
                    case CorpusDomain.INCLUDE_ALBUMFORM_PAGE: {
                        return new IncludeAlbumFormHtmlProducer(parameters, corpus);
                    }
                    case CorpusDomain.INCLUDE_COMMENTFORM_PAGE: {
                        return new IncludeCommentFormHtmlProducer(parameters, corpus);
                    }
                    case CorpusDomain.INCLUDE_CORPUSFORM_PAGE: {
                        return new IncludeCorpusFormHtmlProducer(parameters, corpus);
                    }
                    case CorpusDomain.INCLUDE_SPECIALFORM_PAGE: {
                        return new IncludeSpecialFormHtmlProducer(parameters, corpus);
                    }
                    case CorpusDomain.INCLUDE_THESAURUSFORM_PAGE: {
                        return new IncludeThesaurusFormHtmlProducer(parameters, corpus);
                    }
                    case CorpusDomain.REMOVEDLIST_PAGE: {
                        return new RemovedListHtmlProducer(parameters, corpus);
                    }
                    case CorpusDomain.UI_COMPONENTATTRIBUTESFORM_PAGE: {
                        return new UiComponentAttributesFormHtmlProducer(parameters, corpus);
                    }
                    case CorpusDomain.UI_COMPONENTOPTIONSFORM_PAGE: {
                        return new UiComponentOptionsFormHtmlProducer(parameters, corpus);
                    }
                    case CorpusDomain.UI_COMPONENTPOSITIONFORM_PAGE: {
                        return new UiComponentPositionFormHtmlProducer(parameters, corpus);
                    }
                    default:
                        return null;
                }
            }
            case FICHE: {
                FicheMeta ficheMeta = getFicheMeta(parameters, requestMap, bdfCommandResult);
                switch (page) {
                    case CorpusDomain.FICHE_ADVANCEDCOMMANDS_PAGE: {
                        parameters.checkSubsetAdmin(ficheMeta.getCorpus());
                        return new FicheAdvancedCommandsHtmlProducer(parameters, ficheMeta);
                    }
                    case CorpusDomain.FICHE_REMOVEFORM_PAGE: {
                        parameters.checkSubsetAdmin(ficheMeta.getCorpus());
                        return new FicheRemoveFormHtmlProducer(parameters, ficheMeta);
                    }
                    case CorpusDomain.FICHE_DISCARDFORM_PAGE: {
                        checkWrite(parameters, ficheMeta);
                        return new FicheDiscardFormHtmlProducer(parameters, ficheMeta);
                    }
                    case CorpusDomain.FICHE_RETRIEVEFORM_PAGE: {
                        checkWrite(parameters, ficheMeta);
                        return new FicheRetrieveFormHtmlProducer(parameters, ficheMeta);
                    }
                    case CorpusDomain.FICHE_ADDENDA_PAGE: {
                        checkWrite(parameters, ficheMeta);
                        FichePointeur fichePointeur = toFichePointeur(ficheMeta);
                        FormElementProvider formElementProvider = getFormElementProvider(parameters);
                        UiComponents uiComponents = getUiComponents(bdfServer, requestMap, ficheMeta.getCorpus());
                        return new FicheAddendaHtmlProducer(parameters, fichePointeur, formElementProvider, uiComponents);
                    }
                    case CorpusDomain.FICHE_ALBUM_PAGE: {
                        checkWrite(parameters, ficheMeta);
                        FichePointeur fichePointeur = toFichePointeur(ficheMeta);
                        FormElementProvider formElementProvider = getFormElementProvider(parameters);
                        UiComponents uiComponents = getUiComponents(bdfServer, requestMap, ficheMeta.getCorpus());
                        return new FicheAlbumHtmlProducer(parameters, fichePointeur, formElementProvider, uiComponents);
                    }
                    default:
                        return null;
                }
            }
            default: {
                switch (page) {
                    case CorpusDomain.CORPUS_CREATIONFORM_PAGE: {
                        parameters.checkFichothequeAdmin();
                        return new CorpusCreationFormHtmlProducer(parameters);
                    }
                    case CorpusDomain.DUPLICATIONLOG_PAGE: {
                        SubsetMatch subsetMatch = null;
                        if (bdfCommandResult != null) {
                            subsetMatch = (SubsetMatch) bdfCommandResult.getResultObject(BdfInstructionConstants.SUBSETMATCH_OBJ);
                        }
                        return new DuplicationLogHtmlProducer(parameters, subsetMatch);
                    }
                    case CorpusDomain.REPONDERATIONLOG_PAGE: {
                        ReponderationLog reponderationLog = null;
                        if (bdfCommandResult != null) {
                            reponderationLog = (ReponderationLog) bdfCommandResult.getResultObject(BdfInstructionConstants.REPONDERATIONLOG_OBJ);
                        }
                        return new ReponderationLogHtmlProducer(parameters, reponderationLog);
                    }
                    default:
                        return null;
                }
            }

        }
    }

    private static int getPageType(String page) {
        switch (page) {
            case CorpusDomain.CORPUS_PAGE:
            case CorpusDomain.CORPUS_ADVANCEDCOMMANDS_PAGE:
            case CorpusDomain.CORPUS_DIAGRAM_PAGE:
            case CorpusDomain.CORPUS_PHRASESFORM_PAGE:
            case CorpusDomain.FIELD_CREATIONFORM_PAGE:
            case CorpusDomain.FIELD_OPTIONSFORM_PAGE:
            case CorpusDomain.FIELD_REMOVEFORM_PAGE:
            case CorpusDomain.INCLUDE_ADDENDAFORM_PAGE:
            case CorpusDomain.INCLUDE_ALBUMFORM_PAGE:
            case CorpusDomain.INCLUDE_COMMENTFORM_PAGE:
            case CorpusDomain.INCLUDE_CORPUSFORM_PAGE:
            case CorpusDomain.INCLUDE_SPECIALFORM_PAGE:
            case CorpusDomain.INCLUDE_THESAURUSFORM_PAGE:
            case CorpusDomain.UI_COMPONENTATTRIBUTESFORM_PAGE:
            case CorpusDomain.UI_COMPONENTOPTIONSFORM_PAGE:
            case CorpusDomain.UI_COMPONENTPOSITIONFORM_PAGE:
            case CorpusDomain.REMOVEDLIST_PAGE:
                return CORPUS_ADMIN;
            case CorpusDomain.FICHE_ADVANCEDCOMMANDS_PAGE:
            case CorpusDomain.FICHE_REMOVEFORM_PAGE:
            case CorpusDomain.FICHE_DISCARDFORM_PAGE:
            case CorpusDomain.FICHE_RETRIEVEFORM_PAGE:
            case CorpusDomain.FICHE_ADDENDA_PAGE:
            case CorpusDomain.FICHE_ALBUM_PAGE:
                return FICHE;
            default:
                return 0;
        }
    }

    private static Corpus getCorpus(BdfServer bdfServer, RequestMap requestMap, BdfCommandResult bdfCommandResult) throws BdfInstructionException {
        Corpus corpus = null;
        if (bdfCommandResult != null) {
            corpus = (Corpus) bdfCommandResult.getResultObject(BdfInstructionConstants.CORPUS_OBJ);
        }
        if (corpus == null) {
            corpus = BdfInstructionUtils.getMandatoryCorpus(bdfServer.getFichotheque(), requestMap);
        }
        return corpus;
    }

    private static FicheMeta getFicheMeta(BdfParameters bdfParameters, RequestMap requestMap, BdfCommandResult bdfCommandResult) throws BdfInstructionException {
        return EditionHtmlProducerFactory.getFicheMeta(bdfParameters, requestMap, bdfCommandResult);
    }

    private static UiComponents getUiComponents(BdfServer bdfServer, RequestMap requestMap, Corpus corpus) {
        UiManager uiManager = bdfServer.getUiManager();
        return uiManager.getMainUiComponents(corpus);
    }

    private static FichePointeur toFichePointeur(FicheMeta ficheMeta) {
        FichePointeur fichePointeur = PointeurFactory.newFichePointeur(ficheMeta.getCorpus());
        fichePointeur.setCurrentSubsetItem(ficheMeta);
        return fichePointeur;
    }

    private static FormElementProvider getFormElementProvider(BdfParameters bdfParameters) {
        return FormElementProviderFactory.newInstance(FicheFormParametersBuilder.build(bdfParameters)
                .toFicheFormParameters());
    }

    private static void checkWrite(BdfParameters bdfParameters, FicheMeta ficheMeta) throws PermissionBdfInstructionException {
        PermissionUtils.checkWrite(bdfParameters.getPermissionSummary(), ficheMeta);
    }

}
