/* BdfServer_JsonProducers - Copyright (c) 2012-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.jsonproducers.main;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfCommandResult;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.instruction.OutputParameters;
import fr.exemole.bdfserver.api.interaction.InteractionConstants;
import fr.exemole.bdfserver.api.interaction.domains.MainDomain;
import fr.exemole.bdfserver.api.managers.TableExportManager;
import fr.exemole.bdfserver.api.roles.PermissionSummary;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.jsonproducers.ResultJsonProducer;
import fr.exemole.bdfserver.tools.BdfServerUtils;
import fr.exemole.bdfserver.tools.exportation.table.BdfTableExportUtils;
import fr.exemole.bdfserver.tools.exportation.table.CellEngineUtils;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import fr.exemole.bdfserver.tools.users.BdfUserUtils;
import net.fichotheque.Subset;
import net.fichotheque.exportation.table.CellEngine;
import net.fichotheque.exportation.table.SubsetTable;
import net.fichotheque.exportation.table.TableExport;
import net.fichotheque.exportation.table.TableExportContext;
import net.fichotheque.exportation.table.TableExportDef;
import net.fichotheque.extraction.ExtractionContext;
import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.json.JsonProperty;
import net.mapeadores.util.localisation.MessageLocalisation;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public final class MainJsonProducerFactory {

    private MainJsonProducerFactory() {
    }

    public static JsonProducer getJsonProducer(OutputParameters parameters) throws BdfInstructionException {
        String json = parameters.getOutput();
        BdfCommandResult bdfCommandResult = parameters.getBdfCommandResult();
        BdfServer bdfServer = parameters.getBdfServer();
        RequestMap requestMap = parameters.getRequestMap();
        BdfUser bdfUser = parameters.getBdfUser();
        MessageLocalisation messageLocalisation = bdfServer.getL10nManager().getMessageLocalisation(bdfUser);
        ResultJsonProducer jsonProducer = new ResultJsonProducer(bdfCommandResult, messageLocalisation);
        switch (json) {
            case MainDomain.AVAILABLEPATTERNS_JSON: {
                jsonProducer.add(new PatternArrayMapJsonProperty(bdfServer, bdfUser));
                return jsonProducer;
            }
            case MainDomain.TABLE_JSON: {
                jsonProducer.add(getTableJsonProperty(bdfServer, bdfUser, parameters.getPermissionSummary(), requestMap));
                return jsonProducer;
            }
            default:
                return null;
        }
    }

    private static JsonProperty getTableJsonProperty(BdfServer bdfServer, BdfUser bdfUser, PermissionSummary permissionSummary, RequestMap requestMap) throws BdfInstructionException {
        Subset subset = BdfInstructionUtils.getMandatorySubset(bdfServer.getFichotheque(), requestMap);
        if (!permissionSummary.hasAccess(subset)) {
            return null;
        }
        TableExportManager manager = bdfServer.getTableExportManager();
        SubsetTable subsetTable = null;
        TableExportDef tableExportDef = null;
        String tableExportName = requestMap.getParameter(InteractionConstants.TABLEEXPORT_PARAMNAME);
        if ((tableExportName != null) && (tableExportName.length() > 0)) {
            TableExport tableExport = manager.getTableExport(tableExportName);
            if (tableExport != null) {
                tableExportDef = tableExport.getTableExportDef();
                subsetTable = tableExport.getSubsetTable(subset.getSubsetKey());
            }
            if (subsetTable == null) {
                throw new BdfInstructionException(LogUtils.error("_ tableexport_err_notableexportcontent", tableExportName, subset.getSubsetKey().toString()));
            }
        } else {
            subsetTable = BdfTableExportUtils.toDefaultSubsetTable(bdfServer, subset, BdfUserUtils.getDefaultFicheTableParameters(bdfUser), permissionSummary);
        }
        TableExportContext tableExportContext = bdfServer.getTableExportContext();
        ExtractionContext extractionContext = BdfServerUtils.initExtractionContextBuilder(bdfServer, bdfUser)
                .setSubsetAccessPredicate(permissionSummary.getSubsetAccessPredicate())
                .toExtractionContext();
        CellEngine cellEngine = CellEngineUtils.newCellEngine(bdfServer, extractionContext, tableExportDef, subsetTable);
        return new TableJsonProperty(bdfServer, bdfUser, subsetTable, tableExportContext, cellEngine);
    }

}
