/* BdfServer_JsonProducers - Copyright (c) 2013-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.jsonproducers.pioche;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.instruction.BdfParameters;
import fr.exemole.bdfserver.api.instruction.OutputParameters;
import fr.exemole.bdfserver.api.interaction.domains.PiocheDomain;
import fr.exemole.bdfserver.api.roles.PermissionSummary;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.tools.BdfServerUtils;
import fr.exemole.bdfserver.tools.externalsource.ExternalSourceUtils;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import fr.exemole.bdfserver.tools.roles.RoleUtils;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import net.fichotheque.Fichotheque;
import net.fichotheque.SubsetKey;
import net.fichotheque.addenda.Addenda;
import net.fichotheque.addenda.Document;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.externalsource.ExternalItem;
import net.fichotheque.externalsource.ExternalSource;
import net.fichotheque.selection.DocumentQuery;
import net.fichotheque.selection.FicheQuery;
import net.fichotheque.selection.FieldQuery;
import net.fichotheque.selection.MotcleQuery;
import net.fichotheque.selection.MotcleSelector;
import net.fichotheque.selection.RedacteurQuery;
import net.fichotheque.selection.SelectionContext;
import net.fichotheque.sphere.Redacteur;
import net.fichotheque.sphere.Sphere;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.tools.selection.DocumentQueryBuilder;
import net.fichotheque.tools.selection.FicheQueryBuilder;
import net.fichotheque.tools.selection.MotcleQueryBuilder;
import net.fichotheque.tools.selection.RedacteurQueryBuilder;
import net.fichotheque.tools.selection.SelectionDOMUtils;
import net.fichotheque.tools.selection.SelectionEngines;
import net.fichotheque.utils.AddendaUtils;
import net.fichotheque.utils.Comparators;
import net.fichotheque.utils.CorpusUtils;
import net.fichotheque.utils.FichothequeUtils;
import net.fichotheque.utils.SelectionUtils;
import net.fichotheque.utils.ThesaurusUtils;
import net.fichotheque.utils.selection.DocumentSelectorBuilder;
import net.fichotheque.utils.selection.FicheSelectorBuilder;
import net.fichotheque.utils.selection.MotcleSelectorBuilder;
import net.fichotheque.utils.selection.RedacteurSelectorBuilder;
import net.mapeadores.util.conditions.Condition;
import net.mapeadores.util.conditions.ConditionsUtils;
import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.localisation.CodeCatalog;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.xml.DOMUtils;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
public final class PiocheJsonProducerFactory {

    private PiocheJsonProducerFactory() {
    }

    public static JsonProducer getJsonProducer(OutputParameters parameters) throws BdfInstructionException {
        String json = parameters.getOutput();
        BdfServer bdfServer = parameters.getBdfServer();
        RequestMap requestMap = parameters.getRequestMap();
        BdfUser bdfUser = parameters.getBdfUser();
        Lang lang = parameters.getWorkingLang();
        try {
            lang = Lang.parse(requestMap.getParameter("lang"));
        } catch (ParseException pe) {
        }
        PermissionSummary permissionSummary = parameters.getPermissionSummary();
        switch (json) {
            case PiocheDomain.DOCUMENT_JSON: {
                return new DocumentArrayJsonProducer(getDocuments(parameters, requestMap));
            }
            case PiocheDomain.FICHE_JSON: {
                return new FicheArrayJsonProducer(getFiches(parameters, requestMap), BdfInstructionUtils.getCellEngine(parameters, requestMap));
            }
            case PiocheDomain.MOTCLE_JSON: {
                return new MotcleArrayJsonProducer(lang, getMotscles(bdfServer, permissionSummary, lang, requestMap), BdfInstructionUtils.getCellEngine(parameters, requestMap));
            }
            case PiocheDomain.EXTERNALITEM_JSON: {
                return new ExternalItemArrayJsonProducer(getExternalItems(bdfServer, lang, requestMap));
            }
            case PiocheDomain.REDACTEUR_JSON: {
                return new RedacteurArrayJsonProducer(getRedacteurs(parameters, requestMap), getDefaultSphereKey(requestMap));
            }
            case PiocheDomain.LANGUE_JSON: {
                return new CodeArrayJsonProducer(bdfServer, bdfUser, getLangCodeSet(bdfServer, bdfUser, requestMap));
            }
            case PiocheDomain.PAYS_JSON: {
                return new CodeArrayJsonProducer(bdfServer, bdfUser, getCountryCodeSet(bdfServer, bdfUser, requestMap));
            }
            case PiocheDomain.FIELD_JSON: {
                Condition condition = getCondition(requestMap);
                FieldArrayJsonProducer fieldArrayJsonProducer = new FieldArrayJsonProducer(bdfServer, bdfUser.getLangPreference(), getFieldType(requestMap));
                fieldArrayJsonProducer.init(condition);
                return fieldArrayJsonProducer;
            }
            default:
                return null;
        }
    }

    private static SubsetKey getDefaultSphereKey(RequestMap requestMap) {
        String param = requestMap.getParameter(PiocheDomain.PARAM_PARAMNAME);
        if (param == null) {
            return null;
        }
        try {
            SubsetKey sphereKey = SubsetKey.parse(param);
            if (sphereKey.isSphereSubset()) {
                return sphereKey;
            } else {
                return null;
            }
        } catch (java.text.ParseException pe) {
            return null;
        }
    }

    private static String getFieldType(RequestMap requestMap) {
        String param = requestMap.getParameter(PiocheDomain.PARAM_PARAMNAME);
        if (param == null) {
            return PiocheDomain.ALL_FIELDTYPE;
        } else {
            return param;
        }
    }


    private static SubsetKey getCroisementSubsetKey(RequestMap requestMap) throws BdfInstructionException {
        String subsetString = requestMap.getParameter(PiocheDomain.CROISEMENT_PARAMNAME);
        if (subsetString == null) {
            return null;
        }
        subsetString = subsetString.trim();
        if (subsetString.length() == 0) {
            return null;
        }
        try {
            SubsetKey subsetKey = SubsetKey.parse(subsetString);
            return subsetKey;
        } catch (ParseException pe) {
            throw new BdfInstructionException(BdfInstructionUtils.wrongParameterValue(PiocheDomain.CROISEMENT_PARAMNAME, subsetString));
        }
    }

    private static Condition getCondition(RequestMap requestMap) throws BdfInstructionException {
        String q = BdfInstructionUtils.getMandatoryParameter(requestMap, PiocheDomain.Q_PARAMNAME);
        return ConditionsUtils.parseSimpleCondition(q);
    }


    private static Collection<ExternalItem> getExternalItems(BdfServer bdfServer, Lang lang, RequestMap requestMap) throws BdfInstructionException {
        String subsetsParam = BdfInstructionUtils.getMandatoryParameter(requestMap, PiocheDomain.SUBSETS_PARAMNAME);
        Map<SubsetKey, Thesaurus> thesaurusMap = getThesaurusMap(requestMap, bdfServer.getFichotheque());
        if (thesaurusMap.isEmpty()) {
            throw new BdfInstructionException(BdfInstructionUtils.wrongParameterValue(PiocheDomain.SUBSETS_PARAMNAME, subsetsParam));
        }
        Thesaurus thesaurus = thesaurusMap.values().iterator().next();
        ExternalSource externalSource = ExternalSourceUtils.getExternalSource(bdfServer, thesaurus);
        if (externalSource == null) {
            throw new BdfInstructionException(BdfInstructionUtils.wrongParameterValue(PiocheDomain.SUBSETS_PARAMNAME, subsetsParam));
        }
        lang = BdfServerUtils.checkLangDisponibility(bdfServer, thesaurus, lang);
        String q = BdfInstructionUtils.getMandatoryParameter(requestMap, PiocheDomain.Q_PARAMNAME);
        if (q.isEmpty()) {
            return ExternalSourceUtils.EXTERNALITEM_EMPTYLIST;
        }
        return externalSource.search(q, lang);
    }

    private static Collection<FicheMeta> getFiches(BdfParameters bdfParameters, RequestMap requestMap) throws BdfInstructionException {
        SelectionContext selectionContext = BdfServerUtils.initSelectionContextBuilder(bdfParameters)
                .setSubsetAccessPredicate(bdfParameters.getPermissionSummary().getSubsetAccessPredicate())
                .setFichePredicate(RoleUtils.getFichePredicate(bdfParameters.getBdfServer().getPermissionManager(), bdfParameters.getBdfUser().getRedacteur()))
                .toSelectionContext();
        Map<SubsetKey, Corpus> corpusMap = getCorpusMap(requestMap, bdfParameters.getFichotheque());
        FicheQuery ficheQuery;
        String xml = requestMap.getParameter(PiocheDomain.XML_PARAMNAME);
        if (xml != null) {
            org.w3c.dom.Document document;
            try {
                document = DOMUtils.parseDocument(xml);
            } catch (SAXException sae) {
                throw new BdfInstructionException(LogUtils.error("_ error.exception.xml.sax", sae.getMessage()));
            }
            Element root = document.getDocumentElement();
            switch (root.getTagName()) {
                case "fiche-query":
                case "fiche-select":
                    ficheQuery = SelectionDOMUtils.getFicheConditionEntry(bdfParameters.getFichotheque(), root).getFicheQuery();
                    if (!corpusMap.isEmpty()) {
                        ficheQuery = derive(ficheQuery, corpusMap);
                    }
                    break;
                default:
                    throw new BdfInstructionException(LogUtils.error("_ error.wrong.xml.root", root.getTagName(), "fiche-query"));
            }
        } else {
            FicheQueryBuilder ficheQueryBuilder = FicheQueryBuilder.init()
                    .addCorpus(corpusMap.keySet());
            String q = requestMap.getParameter(PiocheDomain.Q_PARAMNAME);
            if (q != null) {
                Condition condition = ConditionsUtils.parseSimpleCondition(q);
                if (condition != null) {
                    ficheQueryBuilder.setFieldCondition(condition, FieldQuery.TITRE_SCOPE, null);
                }
            }
            ficheQuery = ficheQueryBuilder.toFicheQuery();
        }
        if (ficheQuery != null) {
            Comparator<FicheMeta> comparator = getComparator(selectionContext, requestMap);
            Collection<FicheMeta> result = SelectionEngines.run(selectionContext, FicheSelectorBuilder.init(selectionContext).add(ficheQuery, null).toFicheSelector(), comparator);
            int limit = getLimit(requestMap);
            if ((limit < 1) || (result.size() <= limit)) {
                return result;
            } else {
                int p = 0;
                List<FicheMeta> list = new ArrayList<FicheMeta>(limit);
                for (FicheMeta ficheMeta : result) {
                    list.add(ficheMeta);
                    p++;
                    if (p == limit) {
                        break;
                    }
                }
                return list;
            }
        } else {
            return CorpusUtils.EMPTY_FICHEMETALIST;
        }
    }


    private static Collection<Motcle> getMotscles(BdfServer bdfServer, PermissionSummary permissionSummary, Lang lang, RequestMap requestMap) throws BdfInstructionException {
        Map<SubsetKey, Thesaurus> thesaurusMap = getThesaurusMap(requestMap, bdfServer.getFichotheque());
        SubsetKey shouldNotCroisementSubsetKey = getCroisementSubsetKey(requestMap);
        if (thesaurusMap.size() == 1) {
            Thesaurus thesaurus = thesaurusMap.values().iterator().next();
            lang = BdfServerUtils.checkLangDisponibility(bdfServer, thesaurus, lang);
        }
        SelectionContext selectionContext = BdfServerUtils.initSelectionContextBuilder(bdfServer, lang)
                .setSubsetAccessPredicate(permissionSummary.getSubsetAccessPredicate())
                .toSelectionContext();
        MotcleQuery motcleQuery;
        String xml = requestMap.getParameter(PiocheDomain.XML_PARAMNAME);
        if (xml != null) {
            org.w3c.dom.Document document;
            try {
                document = DOMUtils.parseDocument(xml);
            } catch (SAXException sae) {
                throw new BdfInstructionException(LogUtils.error("_ error.exception.xml.sax", sae.getMessage()));
            }
            Element root = document.getDocumentElement();
            switch (root.getTagName()) {
                case "motcle-query":
                case "motcle-select":
                    motcleQuery = SelectionDOMUtils.getMotcleConditionEntry(bdfServer.getFichotheque(), root).getMotcleQuery();
                    if (!thesaurusMap.isEmpty()) {
                        motcleQuery = derive(motcleQuery, thesaurusMap);
                    }
                    break;
                default:
                    throw new BdfInstructionException(LogUtils.error("_ error.wrong.xml.root", root.getTagName(), "motcle-query"));
            }
        } else {
            MotcleQueryBuilder motcleQueryBuilder = MotcleQueryBuilder.init()
                    .addThesaurus(thesaurusMap.keySet());
            String q = requestMap.getParameter(PiocheDomain.Q_PARAMNAME);
            if (q != null) {
                Condition condition = ConditionsUtils.parseSimpleCondition(q);
                if (condition != null) {
                    motcleQueryBuilder.setContentCondition(condition, MotcleQuery.WITH_IDALPHA_SCOPE);
                }
            }
            motcleQuery = motcleQueryBuilder.toMotcleQuery();
        }
        if (motcleQuery != null) {
            MotcleSelector motcleSelector = MotcleSelectorBuilder.init(selectionContext).add(motcleQuery, null).toMotcleSelector();
            return SelectionEngines.run(selectionContext, shouldNotCroisementSubsetKey, motcleSelector, getMotcleComparator(selectionContext, motcleSelector));
        } else {
            return ThesaurusUtils.EMPTY_MOTCLELIST;
        }
    }

    private static Collection<Document> getDocuments(BdfParameters bdfParameters, RequestMap requestMap) throws BdfInstructionException {
        Map<SubsetKey, Addenda> addendaMap = getAddendaMap(requestMap, bdfParameters.getFichotheque(), bdfParameters.getPermissionSummary());
        if (addendaMap.isEmpty()) {
            return AddendaUtils.EMPTY_DOCUMENTLIST;
        }
        SelectionContext selectionContext = BdfServerUtils.initSelectionContextBuilder(bdfParameters)
                .setSubsetAccessPredicate(bdfParameters.getPermissionSummary().getSubsetAccessPredicate())
                .toSelectionContext();
        Condition condition = getCondition(requestMap);
        DocumentQuery documentQuery = DocumentQueryBuilder.init()
                .addAddenda(addendaMap.keySet())
                .setNameCondition(condition)
                .toDocumentQuery();
        return SelectionEngines.run(selectionContext, DocumentSelectorBuilder.init(selectionContext).add(documentQuery, null).toDocumentSelector());
    }

    private static Collection<Redacteur> getRedacteurs(BdfParameters bdfParameters, RequestMap requestMap) throws BdfInstructionException {
        Map<SubsetKey, Sphere> sphereMap = getSphereMap(requestMap, bdfParameters.getFichotheque());
        SelectionContext selectionContext = BdfServerUtils.initSelectionContextBuilder(bdfParameters)
                .setSubsetAccessPredicate(bdfParameters.getPermissionSummary().getSubsetAccessPredicate())
                .toSelectionContext();
        Condition condition = getCondition(requestMap);
        RedacteurQuery redacteurQuery = RedacteurQueryBuilder.init()
                .addSphere(sphereMap.keySet())
                .setNomcompletCondition(condition)
                .toRedacteurQuery();
        return SelectionEngines.run(selectionContext, RedacteurSelectorBuilder.init(selectionContext).add(redacteurQuery).toRedacteurSelector());
    }

    private static Set<String> getCountryCodeSet(BdfServer bdfServer, BdfUser bdfUser, RequestMap requestMap) throws BdfInstructionException {
        Condition condition = getCondition(requestMap);
        CodeCatalog codeCatalog = bdfServer.getL10nManager().getCodeCatalog();
        if (condition != null) {
            return codeCatalog.getCountryCodeSet(condition, bdfUser.getLangPreference());
        } else {
            return codeCatalog.getCountryCodeSet();
        }
    }

    private static Set<String> getLangCodeSet(BdfServer bdfServer, BdfUser bdfUser, RequestMap requestMap) throws BdfInstructionException {
        Condition condition = getCondition(requestMap);
        CodeCatalog codeCatalog = bdfServer.getL10nManager().getCodeCatalog();
        if (condition != null) {
            return codeCatalog.getLangCodeSet(condition, bdfUser.getLangPreference());
        } else {
            return codeCatalog.getLangCodeSet();
        }
    }

    private static FicheQuery derive(FicheQuery originFicheQuery, Map<SubsetKey, Corpus> corpusMap) {
        List<SubsetKey> originList = originFicheQuery.getCorpusKeyList();
        if (originList.isEmpty()) {
            return SelectionUtils.derive(originFicheQuery, corpusMap.keySet());
        } else {
            List<SubsetKey> newList = new ArrayList<SubsetKey>();
            for (SubsetKey origin : originList) {
                if (corpusMap.containsKey(origin)) {
                    newList.add(origin);
                }
            }
            return SelectionUtils.derive(originFicheQuery, newList);
        }
    }

    private static MotcleQuery derive(MotcleQuery originMotcleQuery, Map<SubsetKey, Thesaurus> thesaurusMap) {
        List<SubsetKey> originList = originMotcleQuery.getThesaurusKeyList();
        if (originList.isEmpty()) {
            return SelectionUtils.derive(originMotcleQuery, thesaurusMap.keySet());
        } else {
            List<SubsetKey> newList = new ArrayList<SubsetKey>();
            for (SubsetKey origin : originList) {
                if (thesaurusMap.containsKey(origin)) {
                    newList.add(origin);
                }
            }
            return SelectionUtils.derive(originMotcleQuery, newList);
        }
    }

    private static Map<SubsetKey, Corpus> getCorpusMap(RequestMap requestMap, Fichotheque fichotheque) throws BdfInstructionException {
        Map<SubsetKey, Corpus> map = new LinkedHashMap<SubsetKey, Corpus>();
        String subsetsParam = requestMap.getParameter(PiocheDomain.SUBSETS_PARAMNAME);
        if (subsetsParam != null) {
            String[] corpusArray = StringUtils.getTechnicalTokens(subsetsParam, true);
            if (corpusArray.length > 0) {
                for (String corpusName : corpusArray) {
                    Corpus corpus = FichothequeUtils.getCorpus(fichotheque, corpusName);
                    if (corpus != null) {
                        map.put(corpus.getSubsetKey(), corpus);
                    }
                }
                if (map.isEmpty()) {
                    throw new BdfInstructionException(BdfInstructionUtils.wrongParameterValue(PiocheDomain.SUBSETS_PARAMNAME, subsetsParam));
                }
            }
        }
        return map;
    }

    private static Map<SubsetKey, Thesaurus> getThesaurusMap(RequestMap requestMap, Fichotheque fichotheque) throws BdfInstructionException {
        Map<SubsetKey, Thesaurus> map = new LinkedHashMap<SubsetKey, Thesaurus>();
        String subsetsParam = requestMap.getParameter(PiocheDomain.SUBSETS_PARAMNAME);
        if (subsetsParam != null) {
            String[] thesaurusArray = StringUtils.getTechnicalTokens(subsetsParam, true);
            if (thesaurusArray.length > 0) {
                for (String thesaurusName : thesaurusArray) {
                    Thesaurus thesaurus = FichothequeUtils.getThesaurus(fichotheque, thesaurusName);
                    if (thesaurus != null) {
                        map.put(thesaurus.getSubsetKey(), thesaurus);
                    }
                }
                if (map.isEmpty()) {
                    throw new BdfInstructionException(BdfInstructionUtils.wrongParameterValue(PiocheDomain.SUBSETS_PARAMNAME, subsetsParam));
                }
            }
        }
        return map;
    }


    private static Map<SubsetKey, Addenda> getAddendaMap(RequestMap requestMap, Fichotheque fichotheque, PermissionSummary permissionSummary) throws BdfInstructionException {
        String subsetsParam = requestMap.getParameter(PiocheDomain.SUBSETS_PARAMNAME);
        Map<SubsetKey, Addenda> map = new LinkedHashMap<SubsetKey, Addenda>();
        boolean done = false;
        if (subsetsParam != null) {
            String[] addendaArray = StringUtils.getTechnicalTokens(subsetsParam, true);
            if (addendaArray.length > 0) {
                done = true;
                for (String addendaName : addendaArray) {
                    Addenda addenda = FichothequeUtils.getAddenda(fichotheque, addendaName);
                    if ((addenda != null) && (permissionSummary.isSubsetAdmin(addenda.getSubsetKey()))) {
                        map.put(addenda.getSubsetKey(), addenda);
                    }
                }
            }
        }
        if (!done) {
            for (Addenda addenda : fichotheque.getAddendaList()) {
                if (permissionSummary.isSubsetAdmin(addenda.getSubsetKey())) {
                    map.put(addenda.getSubsetKey(), addenda);
                }
            }
        }
        return map;
    }

    private static Map<SubsetKey, Sphere> getSphereMap(RequestMap requestMap, Fichotheque fichotheque) throws BdfInstructionException {
        String subsetsParam = requestMap.getParameter(PiocheDomain.SUBSETS_PARAMNAME);
        Map<SubsetKey, Sphere> map = new LinkedHashMap<SubsetKey, Sphere>();
        if (subsetsParam != null) {
            String[] sphereArray = StringUtils.getTechnicalTokens(subsetsParam, true);
            if (sphereArray.length > 0) {
                for (String sphereName : sphereArray) {
                    Sphere sphere = FichothequeUtils.getSphere(fichotheque, sphereName);
                    if (sphere != null) {
                        map.put(sphere.getSubsetKey(), sphere);
                    }
                }
                if (map.isEmpty()) {
                    throw new BdfInstructionException(BdfInstructionUtils.wrongParameterValue(PiocheDomain.SUBSETS_PARAMNAME, subsetsParam));
                }
            }
        }
        return map;
    }

    private static Comparator<Motcle> getMotcleComparator(SelectionContext selectionContext, MotcleSelector motcleSelector) {
        List<Thesaurus> thesaurusList = motcleSelector.getThesaurusList();
        if (thesaurusList.size() == 1) {
            if (thesaurusList.get(0).isIdalphaType()) {
                return null;
            } else {
                return Comparators.label(selectionContext.getWorkingLang());
            }
        } else {
            return Comparators.idalphaAndLabel(selectionContext.getWorkingLang());
        }
    }

    private static Comparator<FicheMeta> getComparator(SelectionContext selectionContext, RequestMap requestMap) throws BdfInstructionException {
        String sort = requestMap.getParameter(PiocheDomain.SORT_PARAMNAME);
        if ((sort == null) || (sort.isEmpty())) {
            return Comparators.FICHEID_ASC;
        }
        try {
            sort = SelectionUtils.checkSortType(sort);
            return Comparators.getComparator(sort, selectionContext.getWorkingLang());
        } catch (IllegalArgumentException iae) {
            throw new BdfInstructionException(LogUtils.error("_ error.unknown.parametervalue", PiocheDomain.SORT_PARAMNAME, sort));
        }
    }

    private static int getLimit(RequestMap requestMap) {
        String limitString = requestMap.getParameter(PiocheDomain.LIMIT_PARAMNAME);
        if (limitString == null) {
            return -1;
        }
        try {
            return Integer.parseInt(limitString);
        } catch (NumberFormatException nfe) {
            return -1;
        }
    }


}
