/* FichothequeLib_API - Copyright (c) 2006-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.utils.selection;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Predicate;
import net.fichotheque.Fichotheque;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.corpus.fiche.Datation;
import net.fichotheque.corpus.fiche.FicheItem;
import net.fichotheque.corpus.fiche.FicheItems;
import net.fichotheque.corpus.fiche.Geopoint;
import net.fichotheque.corpus.metadata.CorpusField;
import net.fichotheque.corpus.metadata.CorpusMetadata;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.pointeurs.FichePointeur;
import net.fichotheque.selection.CroisementCondition;
import net.fichotheque.selection.FicheQuery;
import net.fichotheque.selection.FieldQuery;
import net.fichotheque.selection.FieldTest;
import net.fichotheque.selection.PeriodCondition;
import net.fichotheque.selection.SelectionContext;
import net.fichotheque.sphere.Redacteur;
import net.fichotheque.sphere.Sphere;
import net.fichotheque.utils.CorpusMetadataUtils;
import net.fichotheque.utils.SphereUtils;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.primitives.DateFilter;
import net.mapeadores.util.primitives.DateFilterFactory;
import net.mapeadores.util.primitives.DateFilterUtils;
import net.mapeadores.util.primitives.FuzzyDate;
import net.mapeadores.util.primitives.Ranges;


/**
 *
 * @author Vincent Calame
 */
class FicheSelectEngine {

    private final static FieldKey[] titreScopeArray = {FieldKey.TITRE};
    private final FicheQuery ficheQuery;
    private final Redacteur[] redacteurArray;
    private final Sphere[] sphereArray;
    private final FieldTest fieldTest;
    private final FieldKey geolocalisationFieldKey;
    private final boolean onCreationDate;
    private final boolean onModificationDate;
    private final List<FieldKey> periodFieldKeyList;
    private final DateFilter dateFilter;
    private final CroisementCondition croisementCondition;
    private final Predicate<FicheMeta> motcleConditionPredicate;

    private FicheSelectEngine(CorpusMetadata corpusMetadata, FicheQuery ficheQuery, CroisementCondition croisementCondition, FieldTest fieldTest, Predicate<FicheMeta> motcleConditionPredicate) {
        this.ficheQuery = ficheQuery;
        this.fieldTest = fieldTest;
        this.croisementCondition = croisementCondition;
        this.motcleConditionPredicate = motcleConditionPredicate;
        Fichotheque fichotheque = corpusMetadata.getCorpus().getFichotheque();
        if (ficheQuery.getRedacteurFilter().equals(FicheQuery.SOME_REDACTEURFILTER)) {
            this.redacteurArray = toRedacteurArray(fichotheque, ficheQuery.getRedacteurGlobalIdList());
            this.sphereArray = toSphereArray(fichotheque, ficheQuery.getSphereKeyList());
        } else {
            this.redacteurArray = null;
            this.sphereArray = null;
        }
        if (ficheQuery.isWithGeoloc()) {
            CorpusField corpusField = corpusMetadata.getGeolocalisationField();
            if (corpusField != null) {
                this.geolocalisationFieldKey = corpusField.getFieldKey();
            } else {
                this.geolocalisationFieldKey = null;
            }
        } else {
            this.geolocalisationFieldKey = null;
        }
        PeriodCondition periodCondition = ficheQuery.getPeriodCondition();
        if (periodCondition != null) {
            this.dateFilter = toDateFilter(periodCondition);
            this.periodFieldKeyList = toPeriodFieldKeyList(corpusMetadata, periodCondition);
            this.onCreationDate = periodCondition.isOnCreationDate();
            this.onModificationDate = periodCondition.isOnModificationDate();
        } else {
            this.dateFilter = null;
            this.periodFieldKeyList = null;
            this.onCreationDate = false;
            this.onModificationDate = false;
        }
    }

    void filter(FichePointeur fichePointeur, FilteredCroisementBuilder builder) {
        if (!isSelected(fichePointeur)) {
            return;
        }
        builder.testCondition(croisementCondition);
    }

    boolean isSelected(FichePointeur fichePointeur) {
        FicheMeta ficheMeta = (FicheMeta) fichePointeur.getCurrentSubsetItem();
        switch (ficheQuery.getDiscardFilter()) {
            case FicheQuery.NONE_DISCARDFILTER:
                if (ficheMeta.isDiscarded()) {
                    return false;
                }
                break;
            case FicheQuery.ONLY_DISCARDFILTER:
                if (!ficheMeta.isDiscarded()) {
                    return false;
                }
                break;
        }
        Ranges idRanges = ficheQuery.getIdRanges();
        if (idRanges != null) {
            if (!idRanges.contains(ficheMeta.getId())) {
                return false;
            }
        }
        switch (ficheQuery.getRedacteurFilter()) {
            case FicheQuery.SOME_REDACTEURFILTER:
                if (!isRedacteurSelected(ficheMeta)) {
                    return false;
                }
                break;
            case FicheQuery.ANY_REDACTEURFILTER:
                if (ficheMeta.getRedacteurGlobalIdList().isEmpty()) {
                    return false;
                }
                break;
            case FicheQuery.NONE_REDACTEURFILTER:
                if (!ficheMeta.getRedacteurGlobalIdList().isEmpty()) {
                    return false;
                }
                break;
        }
        if (motcleConditionPredicate != null) {
            if (!motcleConditionPredicate.test(ficheMeta)) {
                return false;
            }
        }
        if (fieldTest != null) {
            if (!fieldTest.success(fichePointeur)) {
                return false;
            }
        }
        if (ficheQuery.isWithGeoloc()) {
            if (geolocalisationFieldKey == null) {
                return false;
            } else {
                Object obj = fichePointeur.getValue(geolocalisationFieldKey);
                if ((obj == null) || (!(obj instanceof Geopoint))) {
                    return false;
                }
            }
        }
        if (dateFilter != null) {
            if (!testDateFilter(fichePointeur, ficheMeta)) {
                return false;
            }
        }
        return true;
    }

    private boolean testDateFilter(FichePointeur fichePointeur, FicheMeta ficheMeta) {
        if (onCreationDate) {
            if (dateFilter.containsDate(ficheMeta.getCreationDate())) {
                return true;
            }
        }
        if (onModificationDate) {
            if (dateFilter.containsDate(ficheMeta.getModificationDate())) {
                return true;
            }
        }
        if (periodFieldKeyList != null) {
            for (FieldKey fieldKey : periodFieldKeyList) {
                Object value = fichePointeur.getValue(fieldKey);
                if (value != null) {
                    if (value instanceof Datation) {
                        if (dateFilter.containsDate(((Datation) value).getDate())) {
                            return true;
                        }
                    } else if (value instanceof FicheItems) {
                        for (FicheItem ficheItem : (FicheItems) value) {
                            if (ficheItem instanceof Datation) {
                                if (dateFilter.containsDate(((Datation) ficheItem).getDate())) {
                                    return true;
                                }
                            }
                        }
                    }
                }
            }
        }
        return false;
    }

    static FicheSelectEngine newInstance(CorpusMetadata corpusMetadata, FicheQuery ficheQuery, CroisementCondition croisementCondition, SelectionContext selectionContext, Predicate<FicheMeta> motcleConditionPredicate) {
        FieldTest fieldTest = null;
        FieldQuery fieldQuery = ficheQuery.getFieldQuery();
        if (fieldQuery != null) {
            short scope = fieldQuery.getScope();
            FieldKey[] fieldKeyArray = null;
            switch (scope) {
                case FieldQuery.TITRE_SCOPE:
                    fieldKeyArray = titreScopeArray;
                    break;
                case FieldQuery.ENTETE_SCOPE:
                    fieldKeyArray = CorpusMetadataUtils.getEnteteFieldKeyArray(corpusMetadata);
                    break;
                case FieldQuery.FICHE_SCOPE:
                    fieldKeyArray = CorpusMetadataUtils.getFicheFieldKeyArray(corpusMetadata);
                    break;
                case FieldQuery.SELECTION_SCOPE:
                    List<FieldKey> fieldKeyList = fieldQuery.getFieldKeyList();
                    fieldKeyArray = fieldKeyList.toArray(new FieldKey[fieldKeyList.size()]);
                    break;
                default:
                    throw new SwitchException("scope: " + scope);
            }
            fieldTest = FieldTestFactory.newFieldTest(fieldQuery.getContentCondition(), selectionContext, fieldKeyArray);
        }
        return new FicheSelectEngine(corpusMetadata, ficheQuery, croisementCondition, fieldTest, motcleConditionPredicate);
    }

    private boolean isRedacteurSelected(FicheMeta ficheMeta) {
        if (sphereArray != null) {
            for (Sphere sphere : sphereArray) {
                if (SphereUtils.ownsToSphere(ficheMeta, sphere)) {
                    return true;
                }
            }
        }
        if (redacteurArray != null) {
            for (Redacteur redacteur : redacteurArray) {
                if (SphereUtils.ownsToRedacteur(ficheMeta, redacteur)) {
                    return true;
                }
            }
        }
        return false;
    }

    private static Redacteur[] toRedacteurArray(Fichotheque fichotheque, List<String> redacteurGlobalIdList) {
        if (redacteurGlobalIdList.isEmpty()) {
            return null;
        }
        List<Redacteur> redacteurList = new ArrayList<Redacteur>();
        for (String globalId : redacteurGlobalIdList) {
            Redacteur redacteur = SphereUtils.getRedacteur(fichotheque, globalId);
            if (redacteur != null) {
                redacteurList.add(redacteur);
            }
        }
        return redacteurList.toArray(new Redacteur[redacteurList.size()]);
    }

    private static Sphere[] toSphereArray(Fichotheque fichotheque, List<SubsetKey> sphereKeyList) {
        if (sphereKeyList.isEmpty()) {
            return null;
        }
        List<Sphere> sphereList = new ArrayList<Sphere>();
        for (SubsetKey sphereKey : sphereKeyList) {
            Sphere sphere = (Sphere) fichotheque.getSubset(sphereKey);
            if (sphere != null) {
                sphereList.add(sphere);
            }
        }
        return sphereList.toArray(new Sphere[sphereList.size()]);
    }

    private static List<FieldKey> toPeriodFieldKeyList(CorpusMetadata corpusMetadata, PeriodCondition periodCondition) {
        List<FieldKey> fieldKeyList = periodCondition.getFieldKeyList();
        if (fieldKeyList.isEmpty()) {
            return null;
        }
        List<FieldKey> list = new ArrayList<FieldKey>();
        for (FieldKey fieldKey : fieldKeyList) {
            if (corpusMetadata.getCorpusField(fieldKey) != null) {
                list.add(fieldKey);
            }
        }
        if (list.isEmpty()) {
            return null;
        }
        return list;
    }

    private static DateFilter toDateFilter(PeriodCondition periodCondition) {
        String startType = periodCondition.getStartType();
        String endType = periodCondition.getEndType();
        if (startType.equals(PeriodCondition.DATE_TYPE)) {
            FuzzyDate startDate = periodCondition.getStartDate();
            switch (endType) {
                case PeriodCondition.SAME_TYPE:
                    return DateFilterFactory.newInstance(startDate);
                case PeriodCondition.DATE_TYPE:
                    return DateFilterFactory.newInstance(startDate, periodCondition.getEndDate());
                case PeriodCondition.ANY_TYPE:
                    return DateFilterFactory.newMinimumFilter(startDate);
                default:
                    throw new SwitchException("Unknown PeriodCondition.getEndType() value: " + endType);
            }
        } else if (startType.equals(PeriodCondition.ANY_TYPE)) {
            switch (endType) {
                case PeriodCondition.SAME_TYPE:
                case PeriodCondition.ANY_TYPE:
                    return DateFilterUtils.NOTNULL_FILTER;
                case PeriodCondition.DATE_TYPE:
                    return DateFilterFactory.newMaximumFilter(periodCondition.getEndDate());
                default:
                    throw new SwitchException("Unknown PeriodCondition.getEndType() value: " + endType);
            }
        } else {
            throw new SwitchException("Unknown PeriodCondition.getStartType() value: " + startType);
        }
    }

}
