/* FichothequeLib_Tools - Copyright (c) 2013-2018 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.importation.parsers;

import java.text.ParseException;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.importation.ParseErrorKeys;
import net.fichotheque.thesaurus.metadata.ThesaurusLangChecker;
import net.fichotheque.thesaurus.policies.DynamicEditPolicies;
import net.fichotheque.tools.corpus.FicheChangeBuilder;
import net.fichotheque.tools.importation.CorpusImportBuilder;
import net.fichotheque.tools.importation.LiensImportBuilder;
import net.fichotheque.tools.importation.ParseResultBuilder;
import net.fichotheque.tools.importation.parsers.handlers.HandlerUtils;
import net.fichotheque.tools.parsers.FicheParser;
import net.mapeadores.util.attr.AttributeChangeBuilder;
import net.mapeadores.util.primitives.FuzzyDate;


/**
 *
 * @author Vincent Calame
 */
public class CorpusCreationRowParser extends RowParser {

    private final CorpusImportBuilder corpusImportBuilder;
    private final ParseResultBuilder parseResultBuilder;
    private final FicheParser.Parameters ficheParserParameters;
    private final CorpusColumns corpusColumns;
    private final DynamicEditPolicies dynamicEditPolicies;
    private final ThesaurusLangChecker thesaurusLangChecker;

    CorpusCreationRowParser(CorpusImportBuilder corpusImportBuilder, ParseResultBuilder parseResultBuilder, FicheParser.Parameters ficheParserParameters, CorpusColumns corpusColumns, DynamicEditPolicies dynamicEditPolicies, ThesaurusLangChecker thesaurusLangChecker) {
        this.corpusImportBuilder = corpusImportBuilder;
        this.parseResultBuilder = parseResultBuilder;
        this.ficheParserParameters = ficheParserParameters;
        this.corpusColumns = corpusColumns;
        this.dynamicEditPolicies = dynamicEditPolicies;
        this.thesaurusLangChecker = thesaurusLangChecker;
    }

    @Override
    public void parseRow(int rowNumber, Row row) {
        int idIndex = corpusColumns.getIdIndex();
        int columnCount = row.getColumnCount();
        Corpus corpus = corpusImportBuilder.getCorpus();
        int newId = -1;
        if ((idIndex != -1) && (columnCount > idIndex)) {
            String id = row.getColumnValue(idIndex).trim();
            try {
                newId = Integer.parseInt(id);
                FicheMeta ficheMeta = corpus.getFicheMetaById(newId);
                if (ficheMeta != null) {
                    parseResultBuilder.addBdfError(ParseErrorKeys.EXISTING_ID, id, rowNumber);
                    return;
                }
            } catch (NumberFormatException nfe) {
                parseResultBuilder.addBdfError(ParseErrorKeys.NOT_INTEGER, id, rowNumber);
                return;
            }
        }
        FicheChangeBuilder ficheChangeBuilder = HandlerUtils.toFicheChangeBuilder(corpusColumns.getFicheHandlerArray(), row, corpus, ficheParserParameters);
        AttributeChangeBuilder attributeChangeBuilder = HandlerUtils.toAttributeChangeBuilder(corpusColumns.getAttributeHandlerArray(), row);
        LiensImportBuilder liensImportBuilder = HandlerUtils.toLiensImportBuilder(corpusColumns.getCroisementHandlers(), row, corpus, ficheParserParameters.getWorkingLang(), dynamicEditPolicies, thesaurusLangChecker);
        FuzzyDate creationDate = null;
        int creationDateIndex = corpusColumns.getCreationDateIndex();
        if ((creationDateIndex != -1) && (columnCount > creationDateIndex)) {
            String creationDateString = row.getColumnValue(creationDateIndex).trim();
            try {
                creationDate = FuzzyDate.parse(creationDateString);
            } catch (ParseException die) {
                parseResultBuilder.addBdfError(ParseErrorKeys.NOT_DATE, creationDateString, rowNumber);
            }
        }
        corpusImportBuilder.addCreation(newId, ficheChangeBuilder, attributeChangeBuilder, liensImportBuilder, creationDate);
    }

    static CorpusCreationRowParser newInstance(String[] fieldArray, CorpusImportBuilder corpusImportBuilder, ParseResultBuilder parseResultBuilder, FicheParser.Parameters ficheParserParameters, DynamicEditPolicies dynamicEditPolicies, ThesaurusLangChecker thesaurusLangChecker) {
        CorpusColumns corpusColumns = CorpusColumns.parse(fieldArray, corpusImportBuilder, parseResultBuilder);
        return new CorpusCreationRowParser(corpusImportBuilder, parseResultBuilder, ficheParserParameters, corpusColumns, dynamicEditPolicies, thesaurusLangChecker);
    }

}
